	LIST P=16C64, F=INHX8M
;
; This program interfaces to a Hitachi (LM032L) 2 line by 20 character display
; module. The program assembles for either 4-bit or 8-bit data interface, depending
; on the value of the 4bit flag. LCD_DATA is the port which supplies the data to
; the LM032L, while LCD_CNTL is the port that has the control lines ( E, RS, R_W ).
; In 4-bit mode the data is transfer on the high nibble of the port ( PORT<7:4> ).
;
;       Program = LM032L.ASM
;       Revision Date:   5-10-94
;
;
       include <C74_reg.h>
       include <lm032l.h>
;
Four_bit         EQU     TRUE       ; Selects 4- or 8-bit data transfers
Data_HI          EQU     FALSE       ; If 4-bit transfers, Hi or Low nibble of PORT
;
;
    if ( Four_bit && !Data_HI )
;
LCD_DATA         EQU     PORTB
LCD_DATA_TRIS    EQU     TRISB
;
    else
;
LCD_DATA         EQU     PORTD
LCD_DATA_TRIS    EQU     TRISD
;
    endif
;
LCD_CNTL         EQU     PORTA
;
;
;
; LCD Display Commands and Control Signal names.
;
    if ( Four_bit && !Data_HI )
;
E				EQU		0				; LCD Enable control line
R_W				EQU		1				; LCD Read/Write control line
RS				EQU		2				; LCD Register Select control line
;
    else
;
E				EQU		3				; LCD Enable control line
R_W				EQU		2				; LCD Read/Write control line
RS				EQU		1				; LCD Register Select control line
;
    endif
;
;
TEMP1       EQU     0x030
;
       org     RESET_V                 ; RESET vector location
RESET       GOTO    START              ;
;
; This is the Periperal Interrupt routine. Should NOT get here
;
    page
        org     ISR_V              ; Interrupt vector location
PER_INT_V	
ERROR1      BCF     STATUS, RP0         ; Bank 0
            BSF     PORTC, 0
            BCF     PORTC, 0
            GOTO    ERROR1
;
;
;
START                               ; POWER_ON Reset (Beginning of program)
            CLRF    STATUS          ; Do initialization (Bank 0)
            CLRF    INTCON
            CLRF    PIR1
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0x00            ; The LCD module does not like to work w/ weak pull-ups
            MOVWF   OPTION_R        ;
            CLRF    PIE1            ; Disable all peripheral interrupts
            MOVLW   0xFF            ;
            MOVWF   ADCON1          ; Port A is Digital.
;
;
            BCF     STATUS, RP0     ; Bank 0
            CLRF    PORTA           ; ALL PORT output should output Low.
            CLRF    PORTB
            CLRF    PORTC
            CLRF    PORTD
            CLRF    PORTE
            BCF     T1CON, TMR1ON   ; Timer 1 is NOT incrementing
;
            BSF     STATUS, RP0     ; Select Bank 1
            CLRF    TRISA           ; RA5 -  0 outputs
            MOVLW   0xF0            ;
            MOVWF   TRISB           ; RB7 - 4 inputs, RB3 - 0 outputs 
            CLRF    TRISC           ; RC Port are outputs
            BSF     TRISC, T1OSO    ; RC0 needs to be input for the oscillator to function
            CLRF    TRISD           ; RD Port are outputs
            CLRF    TRISE           ; RE Port are outputs
            BSF     PIE1, TMR1IE    ; Enable TMR1 Interrupt
            BSF     OPTION_R, RBPU  ; Disable PORTB pull-ups
            BCF     STATUS, RP0     ; Select Bank 0
;
    page
;
; Initilize the LCD Display Module
;
            CLRF    LCD_CNTL        ; ALL PORT output should output Low.

DISPLAY_INIT
    if ( Four_bit && !Data_HI )
            MOVLW   0x02            ; Command for 4-bit interface low nibble
    endif
;
    if ( Four_bit && Data_HI )
            MOVLW   0x020           ; Command for 4-bit interface high nibble
    endif
;
    if ( !Four_bit )
            MOVLW   0x038           ; Command for 8-bit interface
    endif
;
            MOVWF   LCD_DATA        ;
            BSF     LCD_CNTL, E     ; 
            BCF     LCD_CNTL, E     ;
;
; This routine takes the calculated times that the delay loop needs to
; be executed, based on the LCD_INIT_DELAY EQUate that includes the
; frequency of operation. These uses registers before they are needed to 
; store the time.
;
LCD_DELAY   MOVLW   LCD_INIT_DELAY  ;
            MOVWF   MSD             ; Use MSD and LSD Registers to Initilize LCD
            CLRF    LSD             ;
LOOP2       DECFSZ  LSD             ; Delay time = MSD * ((3 * 256) + 3) * Tcy
            GOTO    LOOP2           ;
            DECFSZ  MSD             ;
END_LCD_DELAY
            GOTO    LOOP2           ;
;
; Command sequence for 2 lines of 5x7 characters
;
CMD_SEQ
;
    if ( Four_bit )
        if ( !Data_HI )
            MOVLW   0X02            ; 4-bit low nibble xfer
        else
            MOVLW   0X020           ; 4-bit high nibble xfer
        endif
;
    else                            ; 8-bit mode
            MOVLW   0X038
    endif
;
            MOVWF   LCD_DATA        ; This code for both 4-bit and 8-bit modes
            BSF     LCD_CNTL, E     ; 
            BCF     LCD_CNTL, E     ;
;
    if ( Four_bit )                 ; This code for only 4-bit mode (2nd xfer)
        if ( !Data_HI )
            MOVLW   0x08            ; 4-bit low nibble xfer 
        else
            MOVLW   0x080           ; 4-bit high nibble xfer
        endif
            MOVWF   LCD_DATA        ;
            BSF     LCD_CNTL, E     ; 
            BCF     LCD_CNTL, E     ;
    endif
;
; Busy Flag should be valid after this point
;
            MOVLW   DISP_ON         ;
            CALL    SEND_CMD        ;
            MOVLW   CLR_DISP        ;
            CALL    SEND_CMD        ;
            MOVLW   ENTRY_INC       ;
            CALL    SEND_CMD        ;
            MOVLW   DD_RAM_ADDR     ;
            CALL    SEND_CMD        ;
;
    page
;
;Send a message the hard way
            movlw   'M'
            call    SEND_CHAR
            movlw   'i'
            call    SEND_CHAR
            movlw   'c'
            call    SEND_CHAR
            movlw   'r'
            call    SEND_CHAR
            movlw   'o'
            call    SEND_CHAR
            movlw   'c'
            call    SEND_CHAR
            movlw   'h'
            call    SEND_CHAR
            movlw   'i'
            call    SEND_CHAR
            movlw   'p'
            call    SEND_CHAR

            movlw   B'11000000'     ;Address DDRam first character, second line
            call    SEND_CMD
	
;Demonstration of the use of a table to output a message
            movlw   0               ;Table address of start of message
dispmsg 
            movwf   TEMP1           ;TEMP1 holds start of message address
            call    Table
            andlw   0FFh            ;Check if at end of message (zero
            btfsc   STATUS,Z        ;returned at end)
            goto    out             
            call    SEND_CHAR       ;Display character
            movf    TEMP1,w         ;Point to next character
            addlw   1
            goto    dispmsg
out
loop
            goto    loop            ;Stay here forever
;
;
INIT_DISPLAY
            MOVLW   DISP_ON             ; Display On, Cursor On
            CALL    SEND_CMD            ; Send This command to the Display Module
            MOVLW   CLR_DISP            ; Clear the Display
            CALL    SEND_CMD            ; Send This command to the Display Module
            MOVLW   ENTRY_INC           ; Set Entry Mode Inc., No shift
            CALL    SEND_CMD            ; Send This command to the Display Module
            RETURN
;
    page
;
;*******************************************************************
;* The LCD Module Subroutines                                      *
;*******************************************************************
;
    if ( Four_bit )      ; 4-bit Data transfers?
;
        if ( Data_HI )   ; 4-bit transfers on the high nibble of the PORT
;
;*******************************************************************
;*SendChar - Sends character to LCD                                *
;*This routine splits the character into the upper and lower       * 
;*nibbles and sends them to the LCD, upper nibble first.           *
;*******************************************************************
;
SEND_CHAR
            MOVWF   CHAR            ;Character to be sent is in W
            CALL    BUSY_CHECK      ;Wait for LCD to be ready
            MOVF    CHAR, w          
            ANDLW   0xF0            ;Get upper nibble
            MOVWF   LCD_DATA        ;Send data to LCD
            BCF     LCD_CNTL, R_W   ;Set LCD to read
            BSF     LCD_CNTL, RS    ;Set LCD to data mode
            BSF     LCD_CNTL, E     ;toggle E for LCD
            BCF     LCD_CNTL, E
            SWAPF   CHAR, w
            ANDLW   0xF0            ;Get lower nibble
            MOVWF   LCD_DATA        ;Send data to LCD
            BSF     LCD_CNTL, E     ;toggle E for LCD
            BCF     LCD_CNTL, E
            RETURN
;
        else             ; 4-bit transfers on the low nibble of the PORT
;
;*******************************************************************
;* SEND_CHAR - Sends character to LCD                              *
;* This routine splits the character into the upper and lower      * 
;* nibbles and sends them to the LCD, upper nibble first.          *
;* The data is transmitted on the PORT<3:0> pins                   *
;*******************************************************************
;
SEND_CHAR
            MOVWF   CHAR            ; Character to be sent is in W
            CALL    BUSY_CHECK      ; Wait for LCD to be ready
            SWAPF   CHAR, W
            ANDLW   0x0F            ; Get upper nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BCF     LCD_CNTL, R_W   ; Set LCD to read
            BSF     LCD_CNTL, RS    ; Set LCD to data mode
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            MOVF    CHAR, W
            ANDLW   0x0F            ; Get lower nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            RETURN
;
        endif
    else
;
;*****************************************************************
;* SEND_CHAR - Sends character contained in register W to LCD    *
;* This routine sends the entire character to the PORT           *
;* The data is transmitted on the PORT<7:0> pins                 *
;*****************************************************************
;
SEND_CHAR
            MOVWF   CHAR            ; Character to be sent is in W
            CALL    BUSY_CHECK      ; Wait for LCD to be ready
            MOVF    CHAR, w          
            MOVWF   LCD_DATA        ; Send data to LCD
            BCF     LCD_CNTL, R_W   ; Set LCD in read mode
            BSF     LCD_CNTL, RS    ; Set LCD in data mode
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            RETURN
;
    endif
;
    page
;
;*******************************************************************
;* SendCmd - Sends command to LCD                                  *
;* This routine splits the command into the upper and lower        * 
;* nibbles and sends them to the LCD, upper nibble first.          *
;* The data is transmitted on the PORT<3:0> pins                   *
;*******************************************************************
;
    if ( Four_bit )      ; 4-bit Data transfers?
;
        if ( Data_HI )   ; 4-bit transfers on the high nibble of the PORT
;
;*******************************************************************
;* SEND_CMD - Sends command to LCD                                 *
;* This routine splits the command into the upper and lower        * 
;* nibbles and sends them to the LCD, upper nibble first.          *
;*******************************************************************

SEND_CMD
            MOVWF   CHAR            ; Character to be sent is in W
            CALL    BUSY_CHECK      ; Wait for LCD to be ready
            MOVF    CHAR,w          
            ANDLW   0xF0            ; Get upper nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BCF     LCD_CNTL,R_W    ; Set LCD to read
            BCF     LCD_CNTL,RS     ; Set LCD to command mode
            BSF     LCD_CNTL,E      ; toggle E for LCD
            BCF     LCD_CNTL,E
            SWAPF   CHAR,w
            ANDLW   0xF0            ; Get lower nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BSF     LCD_CNTL,E      ; toggle E for LCD
            BCF     LCD_CNTL,E
            RETURN
;
        else             ; 4-bit transfers on the low nibble of the PORT
;
SEND_CMD
            MOVWF   CHAR            ; Character to be sent is in W
            CALL    BUSY_CHECK      ; Wait for LCD to be ready
            SWAPF   CHAR, W
            ANDLW   0x0F            ; Get upper nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BCF     LCD_CNTL, R_W   ; Set LCD to read
            BCF     LCD_CNTL, RS    ; Set LCD to command mode
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            MOVF    CHAR, W
            ANDLW   0x0F            ; Get lower nibble
            MOVWF   LCD_DATA        ; Send data to LCD
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            RETURN
;
        endif
    else
;
;**************************************************************
;* SEND_CND - Sends command contained in register W to LCD    *
;* This routine sends the entire character to the PORT        *
;* The data is transmitted on the PORT<7:0> pins              *
;**************************************************************

SEND_CMD
            MOVWF   CHAR            ; Command to be sent is in W
            CALL    BUSY_CHECK      ; Wait for LCD to be ready
            MOVF    CHAR, w          
            MOVWF   LCD_DATA        ; Send data to LCD
            BCF     LCD_CNTL, R_W   ; Set LCD in read mode
            BCF     LCD_CNTL, RS    ; Set LCD in command mode
            BSF     LCD_CNTL, E     ; toggle E for LCD
            BCF     LCD_CNTL, E
            RETURN
;
    endif
;
    page
;
    if ( Four_bit )      ; 4-bit Data transfers?
;
        if ( Data_HI )   ; 4-bit transfers on the high nibble of the PORT
;
;*******************************************************************
;* This routine checks the busy flag, returns when not busy        *
;*  Affects:                                                       *
;*      TEMP - Returned with busy/address                          *
;*******************************************************************
;
BUSY_CHECK
            BSF     STATUS, RP0     ; Select Register page 1
            MOVLW   0xFF            ; Set Port_D for input
            MOVWF   LCD_DATA_TRIS
            BCF     STATUS, RP0     ; Select Register page 0
            BCF     LCD_CNTL, RS    ; Set LCD for Command mode
            BSF     LCD_CNTL, R_W   ; Setup to read busy flag
            BSF     LCD_CNTL, E     ; Set E high
            BCF     LCD_CNTL, E     ; Set E low
            MOVF    LCD_DATA, W     ; Read upper nibble busy flag, DDRam address
            ANDLW   0xF0            ; Mask out lower nibble
            MOVWF   TEMP
            BSF     LCD_CNTL, E     ; Toggle E to get lower nibble
            BCF     LCD_CNTL, E
            SWAPF   LCD_DATA, w     ; Read lower nibble busy flag, DDRam address
            ANDLW   0x0F            ; Mask out upper nibble
            IORWF   TEMP            ; Combine nibbles
            BTFSC   TEMP, 7         ; Check busy flag, high = busy
            GOTO    BUSY_CHECK      ; If busy, check again
            BCF     LCD_CNTL, R_W        
            BSF     STATUS, RP0     ; Select Register page 1
            MOVLW   0x0F
            MOVWF   LCD_DATA_TRIS   ; Set Port_D for output
            BCF     STATUS, RP0     ; Select Register page 0
            RETURN
;
        else             ; 4-bit transfers on the low nibble of the PORT
;
;*******************************************************************
;* This routine checks the busy flag, returns when not busy        *
;*  Affects:                                                       *
;*      TEMP - Returned with busy/address                          *
;*******************************************************************
;
BUSY_CHECK
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0xFF            ; Set PortB for input
            MOVWF   LCD_DATA_TRIS
            BCF     STATUS, RP0     ; Bank 0
            BCF     LCD_CNTL, RS    ; Set LCD for Command mode
            BSF     LCD_CNTL, R_W   ; Setup to read busy flag
            BSF     LCD_CNTL, E     ; Set E high
            BCF     LCD_CNTL, E     ; Set E low
            SWAPF   LCD_DATA, W     ; Read upper nibble busy flag, DDRam address
            ANDLW   0xF0            ; Mask out lower nibble
            MOVWF   TEMP            ;
            BSF     LCD_CNTL, E     ; Toggle E to get lower nibble
            BCF     LCD_CNTL, E
            MOVF    LCD_DATA, W     ; Read lower nibble busy flag, DDRam address
            ANDLW   0x0F            ; Mask out upper nibble
            IORWF   TEMP, F         ; Combine nibbles
            BTFSC   TEMP, 7         ; Check busy flag, high = busy
            GOTO    BUSY_CHECK      ; If busy, check again
            BCF     LCD_CNTL, R_W
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0xF0            ;
            MOVWF   LCD_DATA_TRIS   ; RB7 - 4 = inputs, RB3 - 0 = output
            BCF     STATUS, RP0     ; Bank 0
            RETURN
;
        endif
    else
;
;**************************************************************
;* This routine checks the busy flag, returns when not busy   *
;*  Affects:                                                  *
;*      TEMP - Returned with busy/address                     *
;**************************************************************
;
BUSY_CHECK
            BSF     STATUS,RP0      ; Select Register page 1
            MOVLW   0xFF            ; Set port_D for input
            MOVWF   LCD_DATA_TRIS
            BCF     STATUS, RP0     ; Select Register page 0
            BCF     LCD_CNTL, RS    ; Set LCD for command mode
            BSF     LCD_CNTL, R_W   ; Setup to read busy flag
            BSF     LCD_CNTL, E     ; Set E high
            BCF     LCD_CNTL, E     ; Set E low
            MOVF    LCD_DATA, w     ; Read busy flag, DDram address
            MOVWF   TEMP    
            BTFSC   TEMP, 7         ; Check busy flag, high=busy
            GOTO    BUSY_CHECK 
            BCF     LCD_CNTL, R_W        
            BSF     STATUS, RP0     ; Select Register page 1
            MOVLW   0x00
            MOVWF   LCD_DATA_TRIS   ; Set port_D for output
            BCF     STATUS, RP0     ; Select Register page 0
            RETURN
;
    endif
    page
;
Table
            addwf   PCL       ;Jump to char pointed to in W reg
            retlw   'M'
            retlw   'i'
            retlw   'c'
            retlw   'r'
            retlw   'o'
            retlw   'c'
            retlw   'h'
            retlw   'i'
            retlw   'p'
            retlw   ' '
            retlw   'T'
            retlw   'e'
            retlw   'c'
            retlw   'h'
            retlw   'n'
            retlw   'o'
            retlw   'l'
            retlw   'o'
            retlw   'g'
            retlw   'y'
Table_End
            retlw   0
;
	if ( (Table & 0x0FF) >= (Table_End & 0x0FF) )
       MESSG   "Warning - User Definded: Table Table crosses page boundry in computed jump"
    endif
;



    end




